import gkeepapi

from log import Log
from engine_parent import Engine_Parent
from item import Item
import json

class Engine_GKeep(Engine_Parent):

    def __init__(self, id, engine, db):
        Engine_Parent.__init__(self, id, engine, db)

        # Authenticate
        self.authenticate()


        self.auth = gkeepapi.APIAuth(gkeepapi.Keep.OAUTH_SCOPES)
        self.auth.load(self.credentials['email'], self.master_token, self.credentials['android_id'])

        self.keep = gkeepapi.Keep()
        self.keep.load(self.auth)

    def authenticate(self):

        data = {}

        # Saved creds with refresh token 
        if len(self.engine['data']) == 0:
            
            # Get master_token
            from google_gpsoauth_mastertoken import get_gpsoauth_mastertoken            
            data['master_token'] = get_gpsoauth_mastertoken(self.credentials['email'], self.credentials['android_id'])

            # Save to DB
            json_data = json.dumps(data)
            self.db.edit_engine_data(self.id, json_data)

        else:

            # Load data
            data = json.loads(self.engine['data'])

        # Set master token
        self.master_token = data['master_token']
 

    def list_items(self):
        
        # Get objects
        data = self.keep.all()
                
        items = {}
        for note in data:

            if note.trashed:
                continue

            if note.title == 'Shopping List':
                continue

            item = self.get_item_from_note(note)
            items[item.id] = item

        return items
        
    def get_item_from_note(self, note):

        item = Item()
        item.id = note.id
        item.timestamp = str(note.timestamps.updated.timestamp())
        item.title = note.title
        item.body = note.text

        return item

    def get_note_from_item(self, item, note):
       
        note.title = item.title
        note.text = item.body

        return note

    def delete_item(self, id):

        obj = self.keep.get(id)
        obj.delete()

        self.keep.sync()

    def get_item(self, id):

        obj = self.keep.get(id)

        item = self.get_item_from_note(obj)
    
        return item
    
    def add_item(self, item):

        note = self.keep.createNote()
        self.get_note_from_item(item, note)

        self.keep.sync()

        item = self.get_item_from_note(note)

        return item

    def edit_item(self, id, item):

        note = self.keep.get(id)
        self.get_note_from_item(item, note)
        
        self.keep.sync()
       
        item = self.get_item_from_note(note)

        return item
