import json
import time
from datetime import date
import copy

from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request

from googleapiclient.discovery import build

from log import Log
from engine_parent import Engine_Parent
from item import Item
from google_oauth import Google_OAuth

class Engine_GTasks(Engine_Parent):

    SCOPES = ['https://www.googleapis.com/auth/tasks']
    CREDENTIALS = {"web":{"client_id":"413236439480-25n1r9gqpl8gj4mta8men83m9rb5860k.apps.googleusercontent.com","project_id":"databridge-406908","auth_uri":"https://accounts.google.com/o/oauth2/auth","token_uri":"https://oauth2.googleapis.com/token","auth_provider_x509_cert_url":"https://www.googleapis.com/oauth2/v1/certs","client_secret":"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","redirect_uris":["http://localhost:35000/"],"javascript_origins":["http://localhost:35000"]}}

    items_cache = {}

    def __init__(self, id, engine, db):
        Engine_Parent.__init__(self, id, engine, db)
        
        # Credentials
        if not self.credentials:
            self.credentials = self.CREDENTIALS

        # Authenticate
        self.creds = self.authenticate()

        # Service
        self.service = build('tasks', 'v1', credentials=self.creds)
                
    def set_pair_config(self, config):

        # Get list
        list = config['folder'] if 'folder' in config else ''

        self.get_tasklistid(list)

    def get_tasklistid(self, list):

        # Get task lists
        tasklists = self.service.tasklists().list().execute()['items']
        self.tasklist_id = None
        if list:
            # Search for the task list by name
            for tasklist in tasklists:
                if tasklist['title'] == list:
                    self.tasklist_id = tasklist['id']
                    break
        
            if not self.tasklist_id:
                raise Exception('GTask list not fond {0}'.format(list))

        # Default list (first list)
        else:
            self.tasklist_id = tasklists[0]['id']    

    def authenticate(self):

        creds = None

        # Saved creds with refresh token 
        if len(self.engine['data']) > 0:
            data = json.loads(self.engine['data'])
            creds = Credentials.from_authorized_user_info(data, self.SCOPES)

        # Not valid or blank
        if not creds or not creds.valid:

            # Refresh the token
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())

            # Create new only if fresh start (first run - so it does not open a browser in a headless mode)
            elif not creds:

                # Get OAuth credentials
                oauth = Google_OAuth(self.credentials, self.SCOPES)
                creds = oauth.run()
         
            # Save the credentials to the db
            if creds:
                json_data = creds.to_json()
                self.db.edit_engine_data(self.id, json_data)

        return creds

    def get_item_from_task(self, task):

        item = Item()
        
        item.id = task['id']
        item.timestamp = task['etag']

        if 'title' in task:
            item.title = task['title']

        if 'notes' in task:
            item.body = task['notes']
        
        if 'due' in task:
            ttime = time.strptime(task['due'], '%Y-%m-%dT%H:%M:%S.%fZ')
            item.due = date(ttime.tm_year, ttime.tm_mon, ttime.tm_mday)

        return item

    def get_task_from_item(self, item):

        task = {}

        task['title'] = item.get_title(self.capitalize_title)
        task['notes'] = item.body
        
        if item.due:
            task['due'] = item.due.strftime('%Y-%m-%dT%H:%M:%S.%fZ') 

        return task

    def list_items(self):
        
        # Get tasks
        all_tasks = []
        next_page_token = None
        while True:
            
            # Fetch a page of tasks
            tasks_result = self.service.tasks().list(tasklist=self.tasklist_id, pageToken=next_page_token).execute()

            # Extract tasks from the current page
            tasks = tasks_result.get('items', [])

            # Add tasks to the overall list
            all_tasks.extend(tasks)

            # Check for more pages
            next_page_token = tasks_result.get('nextPageToken')
            if not next_page_token:
                break  # No more pages


        # Get objects
        items = {}
        for task in all_tasks:
            
            item = self.get_item_from_task(task)           
            items[item.id] = item

        # Copy local items cache
        self.items_cache = copy.copy(items)

        return items
    
    def get_item(self, id):

        # Check cache
        if id in self.items_cache:
            item = self.items_cache[id]
            return item

        # Get task
        task = self.service.tasks().get(tasklist=self.tasklist_id, task=id).execute()

        item = self.get_item_from_task(task)

        self.items_cache[item.id] = item

       
        return item
    
    def delete_item(self, id):
        
        if id in self.items_cache:
            del self.items_cache[id]

        self.service.tasks().delete(tasklist=self.tasklist_id, task=id).execute()


    def add_item(self, item):

        taskdata = self.get_task_from_item(item)

        result = self.service.tasks().insert(tasklist=self.tasklist_id, body=taskdata).execute()

        new_item = self.get_item_from_task(result)

        self.items_cache[new_item.id] = new_item

        return new_item

    def edit_item(self, id, item):

        taskdata = self.get_task_from_item(item)
        taskdata['id'] = id

        result = self.service.tasks().update(tasklist=self.tasklist_id, task=id, body=taskdata).execute()

        new_item = self.get_item_from_task(result)

        self.items_cache[new_item.id] = new_item

        return new_item
